#include <stdexcept>
#include <regex.h>
#include "string_ref.hpp"
#if (__cplusplus >= 201103L) || !defined(USE_INTERNAL_REGEX)

inline static void throw_if_error(
	const int code,
	const regex_t & r
) {
	if (code) {
		const size_t errlen = regerror(code, &r, nullptr, 0);
		std::string message(errlen, 0);
		regerror(code, &r, &message[0], errlen);
		throw std::runtime_error(message);
	}
}

template<>
bool basic_string_ref<char>::matches(const char * expression) const {
	regex_t r;
	throw_if_error(regcomp(&r, expression, REG_EXTENDED | REG_NOSUB), r);
	// regexec() requires a null-terminated string
	if (m_value[m_length] != 0) {
		std::string tmp(m_value, m_length);
		const int res = regexec(&r, tmp.data(), 0, nullptr, 0);
		if (res == 0) { return true; }
		if (res != REG_NOMATCH) { throw_if_error(res, r); }
	} else {
		const int res = regexec(&r, m_value, 0, nullptr, 0);
		if (res == 0) { return true; }
		if (res != REG_NOMATCH) { throw_if_error(res, r); }
	}
	return false;
}

#endif
